<?php

namespace Ulmus\Api;

use Ulmus\Adapter\AdapterInterface;

class ConnectionAdapter extends \Ulmus\ConnectionAdapter
{
    public string $name;

    public array $configuration;

    protected AdapterInterface $adapter;

    private array $connection;

    protected mixed $apiResponseCallback;

    protected mixed $apiRequestCallback;

    public function resolveConfiguration(): void
    {
        $connection = $this->configuration['connections'][$this->name] ?? [];

        if (false !== ($adapterName = $connection['adapter'] ?? false)) {
            $this->adapter = $this->instanciateAdapter($adapterName);
        } else {
            $this->adapter = new Adapter\Rest();
        }

        $this->adapter->setup($connection);
    }

    public function getConfiguration(): array
    {
        return $this->configuration['connections'][$this->name];
    }

    /**
     * Connect the adapter
     * @return self
     */
    public function connect(): self
    {
        $this->connection = $this->adapter->connect();

        return $this;
    }

    public function connector(): object
    {
        return isset($this->connection) ? $this->connection : $this->connect()->connection;
    }

    public function adapter(): AdapterInterface
    {
        return $this->adapter;
    }

    protected function instanciateAdapter($name): AdapterInterface
    {
        $class = substr($name, 0, 2) === "\\" ? $name : "\\Ulmus\\Api\\Adapter\\$name";

        return new $class();
    }

    public function apiResponseCallback(callable $callback = null) : callable|null
    {
        return $callback ? $this->apiResponseCallback = $callback : $this->apiResponseCallback ?? fn($resp) => $resp->render();
    }

    public function apiRequestCallback(callable $callback = null) : callable|null
    {
        return $callback ? $this->apiRequestCallback = $callback : $this->apiRequestCallback ?? fn($resp) => $resp->render();
    }
}
