<?php

namespace Ulmus\Api\Request;

use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\StreamInterface;
use Psr\Http\Message\UriInterface;
use Ulmus\Api\Common\HttpHeaderEnum;
use Ulmus\Api\Common\MethodEnum;
use Ulmus\Api\Stream\Stream;
use Ulmus\Api\Common\StreamOutputInterface;
use Ulmus\Api\Common\Uri;

class JsonRequest extends Request implements StreamOutputInterface
{
    public const DEFAULT_JSON_ENCODING_FLAGS = JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_TAG | JSON_UNESCAPED_SLASHES;

    public const DEFAULT_JSON_DECODING_FLAGS = JSON_THROW_ON_ERROR;

    public int $jsonEncodingOptions;

    public function __construct(string|UriInterface $uri, MethodEnum $method, null|StreamInterface $stream = null, array $headers = [],  $jsonEncodingOptions = self::DEFAULT_JSON_ENCODING_FLAGS)
    {
        parent::__construct($uri, $method, $stream, $headers + [ 'Content-Type' => "application/json", 'Accept' => "application/json" ]);

        $this->jsonEncodingOptions = $jsonEncodingOptions;
    }

    public static function fromContent(mixed $content, int $encodingOptions = self::DEFAULT_JSON_ENCODING_FLAGS) : StreamInterface
    {
        if ( false === $json = json_encode($content, $encodingOptions) ) {
            throw new \InvalidArgumentException(sprintf('Json encoding failed with message `%s`.', json_last_error_msg()));
        }

        return static::fromJsonEncoded($json);
    }

    public static function fromJsonEncoded(string $json) : StreamInterface
    {
        $obj = Stream::fromTemp($json);
        $obj->rewind();

        return $obj;
    }

    public static function fromRequest(RequestInterface $request) : static
    {
        return new static($request->getUri(), MethodEnum::from($request->getMethod()), $request->getBody(), $request->getHeaders());
    }

    public function render(): mixed
    {
        $content = $this->getBody()->getContents() ?: "[]";

        return json_decode($content, true, 1024, static::DEFAULT_JSON_DECODING_FLAGS);
    }
}