<?php

namespace Ulmus\Api\Response;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\StreamInterface;
use Ulmus\Api\Common\HttpHeaderEnum;
use Ulmus\Api\Stream\Stream;
use Ulmus\Api\Common\StreamOutputInterface;

class JsonResponse extends Response implements StreamOutputInterface
{
    public const DEFAULT_JSON_ENCODING_FLAGS = JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_TAG | JSON_UNESCAPED_SLASHES;

    public const DEFAULT_JSON_DECODING_FLAGS = JSON_THROW_ON_ERROR;

    public int $jsonEncodingOptions;

    public function __construct(null|StreamInterface $stream = null, int $statusCode = 200, array $headers = [], $jsonEncodingOptions = JsonResponse::DEFAULT_JSON_ENCODING_FLAGS)
    {
        parent::__construct($stream, $statusCode, HttpHeaderEnum::normalizeHeaderArray($headers) + [ 'Content-Type' => 'application/json' ]);

        $this->jsonEncodingOptions = $jsonEncodingOptions;
    }

    public static function fromContent(mixed $content, int $encodingOptions = self::DEFAULT_JSON_ENCODING_FLAGS) : StreamInterface
    {
        if ( false === $json = json_encode($content, $encodingOptions) ) {
            throw new \InvalidArgumentException(sprintf('Json encoding failed with message `%s`.', json_last_error_msg()));
        }

        return static::fromJsonEncoded($json);
    }

    public static function fromJsonEncoded(string $json) : StreamInterface
    {
        $obj = Stream::fromTemp($json);
        $obj->rewind();

        return $obj;
    }

    public static function fromResponse(ResponseInterface $response) : static
    {
        return new static($response->getBody(), $response->getStatusCode(), $response->getHeaders());
    }

    public function render(): mixed
    {
        return json_decode($this->getBody()->getContents(), true, 1024, static::DEFAULT_JSON_DECODING_FLAGS);
    }
}